@@ -10,6 +10,11 @@ from django_otp import Device
 from django_otp.oath import totp
 from django_otp.util import hex_validator, random_hex
 
+try:
+    import yubiotp
+except ImportError:
+    yubiotp = None
+
 from .gateways import make_call, send_sms
 
 
@@ -35,9 +40,17 @@ def get_available_phone_methods():
     return methods
 
 
+def get_available_yubikey_methods():
+    methods = []
+    if yubiotp and 'otp_yubikey' in settings.INSTALLED_APPS:
+        methods.append(('yubikey', _('YubiKey')))
+    return methods
+
+
 def get_available_methods():
     methods = [('generator', _('Token generator'))]
     methods.extend(get_available_phone_methods())
+    methods.extend(get_available_yubikey_methods())
     return methods
 
 
@@ -58,21 +71,46 @@ class PhoneDevice(Device):
     method = models.CharField(max_length=4, choices=PHONE_METHODS,
                               verbose_name=_('method'))
 
+    def __repr__(self):
+        return '<PhoneDevice(number={!r}, method={!r}>'.format(
+            self.number,
+            self.method,
+        )
+
+    def __eq__(self, other):
+        if not isinstance(other, PhoneDevice):
+            return False
+        return self.number == other.number \
+            and self.method == other.method \
+            and self.key == other.key
+
     @property
     def bin_key(self):
         return unhexlify(self.key.encode())
 
     def verify_token(self, token):
+        # local import to avoid circular import
+        from two_factor.utils import totp_digits
+
+        try:
+            token = int(token)
+        except ValueError:
+            return False
+
         for drift in range(-5, 1):
-            if totp(self.bin_key, drift=drift) == token:
+            if totp(self.bin_key, drift=drift, digits=totp_digits()) == token:
                 return True
         return False
 
     def generate_challenge(self):
+        # local import to avoid circular import
+        from two_factor.utils import totp_digits
+
         """
         Sends the current TOTP token to `self.number` using `self.method`.
         """
-        token = '%06d' % totp(self.bin_key)
+        no_digits = totp_digits()
+        token = str(totp(self.bin_key, digits=no_digits)).zfill(no_digits)
         if self.method == 'call':
             make_call(device=self, token=token)
         else:
